/*************************************************************************
 *
 *
 *    (c) Copyright Olimex 2009
 *
 *    File name   : extcon_test.h
 *    Description : External connectors test
 *
 *    History :
 *    1. Date        : 10, November 2009
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/

#include <stdio.h>

#include "includes.h"
#include "extcon_test.h"
#include "LED_Test.h"

void Dly100us(void *arg);

#define ARRAY_NUMELEM(A)	(sizeof((A))/sizeof((A)[0]))
#define GET_PIN(pin) (1 << (pin))

// definitions for the tested pins
	PORTS_USED(PortsUsed);
	PORTS_ENABLE_BITS_USED(PortsEnBits);
	MAIN_PIN(MainPin);
	PIN_MAP(PinMap);
	PORTS_NAMES(PortsNames);
	EXT_CON_NAMES(ExtConNames);
	
	typedef enum _TEST_TYPE {
		   TEST_NONE = 0,
		   TEST_ALL_SET,
		   TEST_ALL_CLEAR,
		   TEST_MOVING_ZERO,
		   TEST_MOVING_ONE,
		
		   TEST_FINISHED				
	} TEST_TYPE;
	
static TEST_TYPE testType;

// wait for some time
void WaitSomeTime(void) {
	for(int i = 0; i < 10000; i++);
}

// make output
static void MakePinInput(const PIN* pPin)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin =  GET_PIN(pPin->GPIO_Pin);
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(pPin->GPIO_Port, &GPIO_InitStructure);
}

// make input
static void MakePinOutput(const PIN* pPin)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin =  GET_PIN(pPin->GPIO_Pin);
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(pPin->GPIO_Port, &GPIO_InitStructure);
}

// write 1 to output
static void SetPin(const PIN *pPin)
{
	GPIO_SetBits(pPin->GPIO_Port, GET_PIN(pPin->GPIO_Pin));	
}

// write 0 to output
static void ClearPin(const PIN *pPin)
{
	GPIO_ResetBits(pPin->GPIO_Port, GET_PIN(pPin->GPIO_Pin));
}

// read input pin
uint8_t ReadPin(const PIN *pPin)
{
	return GPIO_ReadInputDataBit(pPin->GPIO_Port, GET_PIN(pPin->GPIO_Pin));
}

// Initial configuration of the test
static void ExtConConfigure(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	
	// enable change of GPIOx configuration
	for(int i = 0; i < ARRAY_NUMELEM(PortsEnBits); i++) {
		   RCC_APB2PeriphResetCmd(  PortsEnBits[i], DISABLE);
		   RCC_APB2PeriphClockCmd(  PortsEnBits[i], ENABLE);
	}
	
	// make main pin output
	GPIO_InitStructure.GPIO_Pin =  GET_PIN(MainPin.GPIO_Pin);
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(MainPin.GPIO_Port, &GPIO_InitStructure);
	
	// make all the other pins inputs
	for(int i = 0; i < ARRAY_NUMELEM(PinMap); i++)
		MakePinInput(PinMap + i);
	
}

int GetPortIndex(GPIO_TypeDef *GPIOx)
{
	
  if (GPIOx == GPIOA)
  {
    return 0;
  }
  else if (GPIOx == GPIOB)
  {
    return 1;
  }
  else if (GPIOx == GPIOC)
  {
    return 2;
  }
  else if (GPIOx == GPIOD)
  {
    return 3;
  }
  else if (GPIOx == GPIOE)
  {
    return 4;
  }
  else if (GPIOx == GPIOF)
  {
    return 5;
  }
  else if (GPIOx == GPIOG)
  {
    return 6;
  }

  return 0;
}


// the test
int ExtConTest(void)
{
	int i;
	int errorFlag = 0, allTestErrFlag = 0;
	
	printf("\n\nExternal connector tests started...\n");

	
	ExtConConfigure();
	
	testType = TEST_ALL_SET;
	while(testType != TEST_FINISHED) {
		errorFlag = 0;
		switch(testType) {
		case TEST_ALL_SET:
			printf("\nTest ALL_SET started...");
			
			// set the main pin
			SetPin(&MainPin);
			
			// wait for some time
			WaitSomeTime();
			
			// check the main pin
			if( !ReadPin(&MainPin) ) {
				errorFlag = -1;
				// report error
				printf("\nERR: Main pin is 0, should be 1");
			}
			
			// check all the other pins if set
			for(i = 0; i < ARRAY_NUMELEM(PinMap); i++) {
				if( !ReadPin(PinMap + i) ) {
					errorFlag = -1;
					// report error
					printf("\nERR: Pin %d, %s (%s, pin %d) is 0, should be 1",
						  PinMap[i].GPIO_Pin, PortsNames[GetPortIndex(PinMap[i].GPIO_Port)],
						  ExtConNames[PinMap[i].EXT_Con], PinMap[i].EXT_Pin);
				}
			}
			if(!errorFlag) {
				// report test ok
				printf(" PASSED.");
			}
			
			testType++;
			break;
			
		case TEST_ALL_CLEAR:
	   		printf("\nTest ALL_CLEAR started...");
			
			// clear the main pin
			ClearPin(&MainPin);
			
			// wait for some time
			WaitSomeTime();
			
			// check the main pin
			if( ReadPin(&MainPin) ) {
				errorFlag = -1;
				// report error
				printf("\nERR: Main pin is 1, should be 0");
			}
			
			// check all the other pins if clear
			for(i = 0; i < ARRAY_NUMELEM(PinMap); i++) {
				if( ReadPin(PinMap + i) ) {
					errorFlag = -1;
					// report error
					printf("\nERR: Pin %d, %s (%s, pin %d) is 1, should be 0",
						  PinMap[i].GPIO_Pin, PortsNames[GetPortIndex(PinMap[i].GPIO_Port)],
						  ExtConNames[PinMap[i].EXT_Con], PinMap[i].EXT_Pin);

				}
			}
			if(!errorFlag) {
				// report test ok
   				printf(" PASSED.");
			}
			
			testType++;		
			break;
			
		case TEST_MOVING_ZERO:
   	   		printf("\nTest MOVING_ZERO started...");

			// set the main pin
			SetPin(&MainPin);
						
			// check all the other pins
			for(i = 0; i < ARRAY_NUMELEM(PinMap); i++) {
				// make output and clear a pin
				MakePinOutput(PinMap + i);
				ClearPin(PinMap + i);

				// wait for some time
				WaitSomeTime();
					
				for(int j = 0; j < ARRAY_NUMELEM(PinMap); j++) {
					// check all the input pins
			   		if( (i != j) && !ReadPin(PinMap + j) ) {
						errorFlag = -1;
						// report error
						printf("\nERR: Pin %d, %s (%s, pin %d) is 0, should be 1",
							  PinMap[j].GPIO_Pin, PortsNames[GetPortIndex(PinMap[j].GPIO_Port)],
							  ExtConNames[PinMap[j].EXT_Con], PinMap[j].EXT_Pin);
					}
				}
					
				// check the main pin
				if( !ReadPin(&MainPin) ) {
					errorFlag = -1;
					// report error
					printf("\nERR: Main pin is 0, should be 1");
				}

				// check the cleared pin
				if( ReadPin(PinMap + i) ) {
					errorFlag = -1;
					// report error
					printf("\nERR: Pin %d, %s (%s, pin %d) is 1, should be 0",
						  PinMap[i].GPIO_Pin, PortsNames[GetPortIndex(PinMap[i].GPIO_Port)],
						  ExtConNames[PinMap[i].EXT_Con], PinMap[i].EXT_Pin);

				}
				
				// make an input again
				MakePinInput(PinMap + i);
				
			}
			if(!errorFlag) {
				// report test ok
				printf(" PASSED.");
			}
			testType++;
			break;
			
		case TEST_MOVING_ONE:
    	   		printf("\nTest MOVING_ONE started...");

			// clear the main pin
			ClearPin(&MainPin);
						
			// check all the other pins
			for(i = 0; i < ARRAY_NUMELEM(PinMap); i++) {
				// make output and set a pin
				MakePinOutput(PinMap + i);
				SetPin(PinMap + i);

				// wait for some time
				WaitSomeTime();
					
				for(int j = 0; j < ARRAY_NUMELEM(PinMap); j++) {
					// check all the input pins
			   		if( (i != j) && ReadPin(PinMap + j) ) {
						errorFlag = -1;
						// report error
						printf("\nERR: Pin %d, %s (%s, pin %d) is 1, should be 0",
							  PinMap[j].GPIO_Pin, PortsNames[GetPortIndex(PinMap[j].GPIO_Port)],
							  ExtConNames[PinMap[j].EXT_Con], PinMap[j].EXT_Pin);

					}
				}

				// check the main pin
				if( ReadPin(&MainPin) ) {
					errorFlag = -1;
					// report error
					printf("\nERR: Main pin is 0, should be 1");
				}

				// check the set pin
				if( !ReadPin(PinMap + i) ) {
					errorFlag = -1;
					// report error
					printf("\nERR: Pin %d, %s (%s, pin %d) is 0, should be 1",
						  PinMap[i].GPIO_Pin, PortsNames[GetPortIndex(PinMap[i].GPIO_Port)],
						  ExtConNames[PinMap[i].EXT_Con], PinMap[i].EXT_Pin);

				}
				
				
				// make an input again
				MakePinInput(PinMap + i);
				
			}
			if(!errorFlag) {
				// report test ok
  				printf(" PASSED.");
			}

			testType++;
			break;
		}
		
		allTestErrFlag |= errorFlag;
	}
	
	// all tests finished
	if(!allTestErrFlag) {
		printf("\nAll tests PASSED SUCCESSFULLY!!!");
		printf("\nAll tests PASSED SUCCESSFULLY!!!");
		printf("\nAll tests PASSED SUCCESSFULLY!!!");
		printf("\nAll tests PASSED SUCCESSFULLY!!!");
	}
	else	{
		printf("\nERROR during tests!!!");
		printf("\nERROR during tests!!!");
		printf("\nERROR during tests!!!");
		printf("\nERROR during tests!!!");
	}
		
	MakePinInput(&MainPin);
	
	
	return (allTestErrFlag) ? -1 : 0;
}

void ExtConMainPinDisable(void)
{
	MakePinInput(&MainPin);
}

void ExtConMainPinEnable(void)
{
	MakePinOutput(&MainPin);
	ClearPin(&MainPin);
}



